<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Form\Facades\Form;
use EventPortal\Domain\Table\Facades\Table;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\Render;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Database\Query\JoinClause;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Inertia\Response;

class UserController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:user.show')->only('show');
        $this->middleware('check-permission:user.store')->only(['create', 'store']);
        $this->middleware('check-permission:user.update')->only(['edit', 'update']);
        $this->middleware('check-permission:user.destroy')->only('delete');
    }

    public function index(): Response
    {
        $query = Admin::query()
            ->join('users', function (JoinClause $query) {
                return $query->on('admins.user_id', '=', 'users.id');
            });

        $table = Table::make('admin.index')
            ->query($query)
            ->request(request())
            ->get();

        $quickButton = Button::make('Create new user')
            ->method(InertiaMethod::VISIT())
            ->icon('plus')
            ->route('admin.user.create');

        return Render::component('User/Index')
            ->key('user.index')
            ->withTable($table)
            ->withQuickButton($quickButton)
            ->get();
    }

    public function create(): Response
    {
        $form = Form::make('admin.create')->model(new Admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add('Create a user')
            ->get();

        return Render::component('User/Create')
            ->key('user.index')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->get();
    }

    public function store(Request $request): RedirectResponse
    {
        $payload = Form::make('admin.create')
            ->model(new Admin)
            ->validate($request->all())
            ->save($request->all());

        toast()->success(trans('user.created'));

        return redirect()->route('admin.user.show', ['admin' => $payload->getModel()]);
    }

    public function edit(Admin $admin): Response
    {
        $form = Form::make('admin.edit')->model($admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add($admin->full_name, route('admin.user.show', $admin->id))
            ->get();

        return Render::component('User/Create')
            ->key('user.edit')
            ->withForm($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle("Edit {$admin->full_name}")
            ->get();
    }

    public function update(Request $request, Admin $admin): RedirectResponse
    {
        Form::make('admin.edit')
            ->model($admin)
            ->validate($request->all())
            ->save($request->all());

        toast()->success(trans('user.updated'));

        return redirect()->route('admin.user.show', $admin);
    }

    public function show(Admin $admin): Response
    {
        $form = Form::make('admin.show')->model($admin)->render();

        $breadcrumbs = Breadcrumb::make()
            ->add('Users', route('admin.user.index'))
            ->add($admin->full_name)
            ->get();

        return Render::component('User/Show')
            ->key('user.show')
            ->withData($form)
            ->withBreadcrumbs($breadcrumbs)
            ->withTitle("View {$admin->full_name}")
            ->get();
    }

    public function delete(Admin $admin): RedirectResponse
    {
        $admin->user->delete();
        $admin->delete();

        toast()->success(trans('user.deleted'));

        return redirect()->route('admin.user.index');
    }
}
